<?php
/**
 * WooCommerce AvaTax
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@skyverge.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce AvaTax to newer
 * versions in the future. If you wish to customize WooCommerce AvaTax for your
 * needs please refer to http://docs.woocommerce.com/document/woocommerce-avatax/
 *
 * @author    SkyVerge
 * @copyright Copyright (c) 2016-2022, SkyVerge, Inc.
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

use SkyVerge\WooCommerce\AvaTax\Landed_Cost_Sync_Handler;
use SkyVerge\WooCommerce\PluginFramework\v5_10_14 as Framework;

defined( 'ABSPATH' ) or exit;

/**
 * Set up the admin settings.
 *
 * @since 1.0.0
 */
class WC_AvaTax_Settings {

	/** @var string $id The settings page ID */
	protected $id = 'avatax';


	/**
	 * Constructs the class.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		$this->add_hooks();
		$this->cache_company_id();
	}

	/**
	 * Adds action and filter hooks.
	 *
	 * @since 1.13.0
	 */
	private function add_hooks() {

		// add the settings section to the WooCommerce Tax tab
		add_filter( 'woocommerce_get_sections_tax', [ $this, 'add_settings_section' ] );

		// output the settings
		add_action( 'woocommerce_get_settings_tax', [ $this, 'add_settings' ] );

		// display a custom address settings field
		add_action( 'woocommerce_admin_field_wc_avatax_address', [ $this, 'display_address_fields' ] );

		//display a custom license key field with landing message
		add_action( 'woocommerce_admin_field_wc_avatax_api_license_key_type', [ $this, 'display_api_license_key_field' ] );
		add_action( 'woocommerce_admin_field_wc_avatax_api_environment_type', [ $this, 'display_api_environment_field' ] );
		
		// display the product sync fields
		add_action( 'woocommerce_admin_field_wc_avatax_product_sync', [ $this, 'display_product_sync_field' ] );

		// save the settings
		add_action( 'woocommerce_settings_save_tax', [ $this, 'save_settings' ] );

		// clears the company ID cache when the company code changes
		add_action( 'update_option_wc_avatax_company_code', [ $this, 'prune_company_id_cache' ], 10, 2 );

		// clears the license key cache when the license key changes
		add_action( 'update_option_wc_avatax_api_license_key', [ $this, 'prune_account_number_cache' ], 9, 2 );

		// clears the API account number cache when the account number changes
		add_action( 'update_option_wc_avatax_api_account_number', [ $this, 'prune_account_number_cache' ], 9, 2 );
		add_action( 'update_option_wc_avatax_api_environment', [ $this, 'prune_account_number_cache' ], 9, 2 );

		// trigger a new sync when the cross-border countries list updates
		add_action( 'update_option_wc_avatax_api_product_countries_sync', [ $this, 'handle_countries_sync_update' ], 10, 2 );
	}

	/**
	 * Add the AvaTax section to the Tax tab.
	 *
	 * @since 1.0.0
	 * @param array $sections The existing Tax sections.
	 * @return array $sections The new Tax sections.
	 */
	public function add_settings_section( $sections ) {

		$sections[ $this->id ] = __( 'AvaTax', 'woocommerce-avatax' );

		return $sections;
	}

	/**
	 * Get the API settings.
	 *
	 * @since 1.0.0
	 * @return array $settings The API settings.
	 */
	public function get_api_settings() {

		$connection_status = get_transient( 'wc_avatax_connection_status' );

		$settings = array(

			array(
				'name' => __( 'Connect to Avalara', 'woocommerce-avatax' ),
				'type' => 'title'
			),
			array(
				'id'      => 'wc_avatax_api_environment',
				'name'    => __( 'Choose your account type', 'woocommerce-avatax' ),
				'options' => array(
					'production'  => __( 'Production', 'woocommerce-avatax' ),
					'development' => __( 'Development', 'woocommerce-avatax' ),
				),
				'desc'=>'Select your Production or Development account.',
				'default' => 'production',
				'type'    => 'wc_avatax_api_environment_type',
			),

			array(
				'id'                => 'wc_avatax_api_account_number',
				'name'              => __( 'Account ID', 'woocommerce-avatax' ),
				'type'              => 'text',
				'class'             => 'wc-avatax-connection-field',
				'css'               => 'min-width:300px;',
				'custom_attributes' => array(
					'data-wc-avatax-connection-status' => $connection_status,
				),
			),

			array(
				'id'                => 'wc_avatax_api_license_key',
				'name'              => __( 'License Key/Password', 'woocommerce-avatax' ),
				'type'              => 'wc_avatax_api_license_key_type',
				'class'             => 'wc-avatax-connection-field',
				'css'               => 'min-width:300px;',
				'custom_attributes' => array(
					'data-wc-avatax-connection-status' => $connection_status,
				),
			),

			array(
				'type' => 'sectionend',
			),
		);

		/**
		 * Filter the API settings.
		 *
		 * @since 1.0.0
		 * @param array $settings The API settings.
		 */
		return (array) apply_filters( 'woocommerce_get_settings_' . $this->id . '_api', $settings );
	}

	/**
	 * Gets the tax calculation settings.
	 *
	 * @since 1.0.0
	 *
	 * @return array $settings the tax calculation settings
	 */
	public function get_tax_settings() {

		$settings = [

			[
				'name' => __( 'Tax Calculation', 'woocommerce-avatax' ),
				'type' => 'title',
			],

			[
				'id'       => 'wc_avatax_enable_tax_calculation',
				'name'     => __( 'Enable tax calculation ', 'woocommerce-avatax' ),
				'default'  => 'yes',
				'type'     => 'checkbox',
				'class'	   => 'checkbox-toggle',
			],
			[
				'id'      => 'wc_avatax_record_calculations',
				'name'    => __( 'Save transactions in Avalara for reporting', 'woocommerce-avatax' ),
				'desc_tip'=> __( 'Turn this on to save transactions in Avalara when an order status in WooCommerce is "Payment received","Processing" or "Complete."', 'woocommerce-avatax' ),
				'default' => 'yes',
				'type'    => 'checkbox',
				'class'	   => 'checkbox-toggle',
			],

			$this->get_company_field(),
			
			/**
			 * Setting added to configure Colorado RDF.
			 * 
			 * @since 2.5.1
			 */
			$this->get_colorado_rdf('US'),
			[
				'id'      => 'wc_avatax_shipping_code',
				'name'    => __( 'Shipping tax code', 'woocommerce-avatax' ),
				'desc'	  => "Set a default shipping tax code for transactions that are missing a shipping tax code. AvaTax assigns the shippping tax code for a third-party common carrier (FR) if you don't enter a value.",
				'type'    => 'text',
				'default' => 'FR',
			],
			[
				'type' => 'sectionend',
			],
		];

		/**
		 * Filters the tax calculation settings.
		 *
		 * @since 2.7.0
		 *
		 * @param array $settings the tax calculation settings
		 */
		return (array) apply_filters( 'woocommerce_get_settings_' . $this->id . '_tax_calculation', $settings );
	}

	/**
	 * Get the company settings.
	 *
	 * @since 2.7.0
	 * @return array $settings The company settings.
	 */
	public function get_company_field() {
		try
		{
			$company_code_list = [];
			if(wc_avatax()->has_api_credentials_set() && wc_avatax()->check_api())
			{
				$company_code_list = wc_avatax()->get_api()->get_companies()->get_company_code_list();
				if(empty($company_code_list))
				{
					return([]);
				}
				if(count($company_code_list)<=1)
				{
					return([
						'id'       => 'wc_avatax_company_code',
						'title'    => __( 'Company name', 'woocommerce-avatax' ),
						'type'     => 'text',
						'default'  => count($company_code_list)==1 ? reset($company_code_list) : "No Company found.",
						'custom_attributes' => array(
							'readonly' => 'readonly',
						),
					]);
				}
				else
				{
					return([
						'id'       => 'wc_avatax_company_code',
						'name'     => __( 'Company name', 'woocommerce-avatax' ),
						'type'     => 'select',
						'default'  => get_option('wc_avatax_company_code'),
						'class'    => 'wc-enhanced-select',
						'options'  => $company_code_list,
					]);
				}
			}
		}
		catch ( Framework\SV_WC_API_Exception $e ){
			wc_avatax()->log("Error while fetching company setting - " . $e->getMessage());
		}
	}

	/**
	 * Get Colorado RDF Settings.
	 *
	 * @since 2.7.0
	 * @param string country
	 * @return array $settings Get Colorado RDF Settings.
	 */
	public function get_colorado_rdf(string $country) {	
		
		if(wc_avatax()->has_api_credentials_set() && wc_avatax()->check_api())
		{
			$supported_states = wc_avatax()->get_landed_cost_handler()->get_supported_states($country);
			if (wc_avatax()->wc_avatax_utilities()->is_colorado_nexus_present())
			{
				return([
					'id'      => 'wc_avatax_enable_co_rdf',
					'name'    => __( 'Calculate Colorado retail delivery fee', 'woocommerce-avatax' ),
					'desc_tip'=> 'Turn this on to calculate the Colorado retail delivery fee on orders in Colorado. Not all businesses are required to collect the retail delivery fee.' ,
					'default' => 'yes',
					'type'    => 'checkbox',
					'class'	  => 'checkbox-toggle',
				]);
			}
			
		}
	}

	/**
	 * Get the address validation settings.
	 *
	 * @since 2.7.0
	 * @return array $settings The address validation settings.
	 */
	public function get_address_settings() {
		if(wc_avatax()->has_api_credentials_set() && wc_avatax()->check_api() && !wc_avatax()->wc_avatax_utilities()->has_nexus_outside_USandCanada())
		{
			$settings = array(

				array(
					'type' => 'title',
					'name' => __( 'Address validation', 'woocommerce-avatax' ),
				),
	
				array(
					'id'       => 'wc_avatax_enable_address_validation',
					'name'     => __( 'Validate address', 'woocommerce-avatax' ),
					'desc_tip' => __( 'Turn this on to add a button during checkout that customers can use to validate and correct addresses in the U.S. and Canada.', 'woocommerce-avatax' ),
					'default'  => 'no',
					'type'     => 'checkbox',
					'class'	   => 'checkbox-toggle',
				),
				array(
					'type' => 'sectionend',
				),
			);
			/**
			 * Filter the address validation settings.
			 *
			 * @since 1.0.0
			 * @param array $settings The address validation settings.
			 */
			return (array) apply_filters( 'woocommerce_get_settings_' . $this->id . '_address_validation', $settings );
		}
		return [];
	}

	/**
	 * Get the address verification settings.
	 *
	 * @since 1.0.0
	 * @return array $settings The address verification settings.
	 */
	public function get_misc_settings() {

		$settings = array(

			array(
				'type' => 'title',
				'name' => __( 'Logs', 'woocommerce-avatax' ),
			),

			array(
				'id'      => 'wc_avatax_debug',
				'name'    => __( 'Enable logging', 'woocommerce-avatax' ),
				'desc_tip'    => __( 'Turn this on to create a log of API requests, responses and errors for transactions. The log is saved in WooCommerce. It\'s useful when troubleshooting the issues with transactions.', 'woocommerce-avatax' ),
				'default' => 'no',
				'type'    => 'checkbox',
				'class'	   => 'checkbox-toggle',
			),

			array(
				'type' => 'sectionend',
			),
		);

		/**
		 * Filter the misc settings.
		 *
		 * @since 2.6.0
		 * @param array $settings The misc settings.
		 */
		return apply_filters( 'woocommerce_get_settings_' . $this->id . '_misc', $settings );
	}

	/**
	 * Get Exemption Certificate Management settings.
	 *
	 * @since 2.7.0
	 * @return array $settings Exemption Certificate Management settings.
	 */
	public function get_ecm_settings() {
		if(wc_avatax()->has_api_credentials_set() && wc_avatax()->check_api() && wc_avatax()->wc_avatax_utilities()->has_ecm_subscription())
		{
			$settings = array(

			array(
				'type' => 'title',
				'name' => __( 'Exemption certificate management', 'woocommerce-avatax' ),
			),

			array(
				'id'       => 'wc_avatax_enable_ecm',
				'name'     => __( 'Enable exemption certificates', 'woocommerce-avatax' ),
				'desc_tip' => __( "Turn this on to add a link during checkout for customers to create or upload exemption certificates.", 'woocommerce-avatax' ),
				'default'  => 'yes',
				'type'     => 'checkbox',
				'class'	   => 'checkbox-toggle',
			),
			array(
				'type' => 'sectionend',
			),
			);
			return (array) apply_filters( 'woocommerce_get_settings_' . $this->id . '_ecm', $settings );
		}
		else
		{
			$resp = [];
			return $resp;
		}
		
	}

	

	/**
	 * Get all of the combined settings.
	 *
	 * @since 1.0.0
	 * @return array $settings The combined settings.
	 */
	public function get_settings() {

		$settings = $this->get_api_settings();

		if ( $this->get_plugin()->check_api() ) {
			$settings = array_merge(
				$settings,
				$this->get_product_sync_settings(),
				$this->get_tax_settings(),
				$this->get_address_settings(),
				$this->get_misc_settings(),
				$this->get_ecm_settings()
			);
		}

		/**
		 * Filter the combined settings.
		 *
		 * @since 1.0.0
		 * @param array $settings The combined settings.
		 */
		return apply_filters( 'woocommerce_get_settings_' . $this->id, $settings );
	}

	/**
	 * Replace core Tax settings with our own when the AvaTax section is being viewed.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function add_settings( $settings ) {

		global $current_section;

		// Output the general settings
		if ( $this->id == $current_section ) {

			// Always display the API settings
			$settings = array_merge(
				$this->get_api_settings(),
			);

			// If the API credentials are good, display the other settings
			if ( $this->get_plugin()->check_api() ) {

				$settings = array_merge(
					$settings,
					$this->get_tax_settings(),
					$this->get_address_settings(),
					$this->get_ecm_settings(),
					$this->get_product_sync_settings(),
					$this->get_misc_settings()
				);
				if ( get_option('wc_avatax_company_code') === "Select" || empty(get_option('wc_avatax_company_code')) ) {
					wc_avatax()->log("Logged error  " . get_option('wc_avatax_company_code'));
					WC_Admin_Settings::add_error( __( 'Avalara Company Code is not selected.', 'woocommerce-avatax' ) );
				}
			}
			

		}

		return $settings;
	}

	/**
	 * Gets the address settings with default fallback values.
	 *
	 * @since 1.7.2
	 *
	 * @param string $setting_id address setting field ID
	 * @return string[] address values
	 */
	private function get_address_values( $setting_id = 'wc_avatax_origin_address' ) {

		$settings         = $this->get_settings();
		$address_defaults = [];
		$address_values   = (array) WC_Admin_Settings::get_option( $setting_id, [] );

		// Loop through each setting to find the address default values.
		foreach ( $settings as $setting ) {

			// The setting ID has to match.
			if ( isset( $setting['id'] ) && $setting_id === $setting['id'] ) {

				$address_defaults = $setting['default'];

				// remove keys that are not present in saved settings
				if ( ! empty( $address_values ) ) {

					// loop through saved address values
					foreach ( $address_defaults as $address_key => $address_value ) {

						if ( ! isset( $address_values[ $address_key ] ) ) {

							unset( $address_defaults[ $address_key ] );
						}
					}
				}

				break;
			}
		}

		return wp_parse_args( $address_values, $address_defaults );
	}
	
	/**
	 * Display a custom address settings field.
	 *
	 * @since 1.0.0
	 * @param array $options The field options.
	 */
	public function display_address_fields( $options ) {

		$description      = WC_Admin_Settings::get_field_description( $options );
		$tooltip_html     = $description['tooltip_html'];
		$description      = $description['description'];
		$id               = $options['id'];
		$label            = $options['title'];
		$type             = $options['type'];
		$values           = $this->get_address_values( $id );
		$countries        = WC()->countries->countries;
		$selected_country = $values['country'] ?? WC()->countries->get_base_country();

		include( $this->get_plugin()->get_plugin_path() . '/src/admin/views/setting-address.php' );
	}

	/**
	 * Saves the settings.
	 *
	 * @internal
	 *
	 * @since 1.0.0
	 *
	 * @global string $current_section The current settings section.
	 */
	public function save_settings() {
		global $current_section;
		$is_connecting = true;
		if ( $this->id === $current_section ) {

			// if the API credentials were good at last check, save the settings
			if ( 'connected' === get_transient( 'wc_avatax_connection_status' ) ) {
				$this->save_fields( $this->get_tax_settings() );
				$this->save_fields( $this->get_address_settings() );
				$this->save_fields( $this->get_ecm_settings() );
				$is_connecting = false;
			}
			else {
				$is_connecting = true;
			}

			// always save the API, product sync & misc settings
			$this->save_fields( $this->get_api_settings() );
			$this->save_fields( $this->get_misc_settings() );
			$this->save_fields( $this->get_product_sync_settings() );

			//$this->get_plugin()->clear_company_id_cache();

			// reset the API status transient
			delete_transient( 'wc_avatax_connection_status' );
			delete_transient( 'wc_avatax_subscribed' );

			// TODO: we should do the same for Item Classification API as well {IT 2022-01-11}
			// check the API again and display an error for bad credentials
			if ( $this->get_plugin()->check_api(false) ) {
				$this->cache_company_id();

				if($is_connecting){
					// save the settings
					add_action( 'woocommerce_settings_saved', [ $this, 'save_default_fields' ] );
				}
				$this->save_origin_address_field();
			}
			else
			{
				if($is_connecting){
					add_action( 'woocommerce_settings_saved', [ $this, 'clear_default_fields' ] );
				}
			}
			
			// stop cross-border product sync in case credentials were removed
			if ( ! $this->get_plugin()->has_hs_api_credentials_set() ) {
				$this->get_plugin()->get_landed_cost_sync_handler()->stop_syncing();
			}
		}
	}
	/**
	 * Save default fields. Enables all visible settings when connecting to AvaTAx.
	 *
	 * @internal
	 *
	 * @since 2.7.0
	 * 
	 */
	public function save_default_fields() {
		wc_avatax()->wc_avatax_utilities()->save_default_fields();
	}

	/**
	 * Clears default fields.
	 *
	 * @internal
	 *
	 * @since 2.7.0
	 * 
	 */
	public function clear_default_fields() {
		wc_avatax()->wc_avatax_utilities()->disconnect_avatax(true);

		if(!$this->get_plugin()->has_api_credentials_set()){
			WC_Admin_Settings::add_error( __( 'Enter Account ID and License key.', 'woocommerce-avatax' ) );
		}
	}

	/**
	 * Save the settings fields.
	 *
	 * This is a simple wrapper for `WC_Admin_Settings::save_fields` to intercept our custom "address"
	 * field type for special handling. All other fields are saved as usual. This is being improved in WC 2.4+
	 * but for now this is easiest for older versions.
	 *
	 * @since 1.0.0
	 *
	 * @param array $fields the settings fields to save
	 */
	private function save_fields( $fields ) {

		// Loop through each setting and look for an address field
		foreach ( $fields as $key => $field ) {

			// If found, save it our way and remove it from the settings to save the WooCommerce way
			if ( isset( $field['id'], $field['type'] ) && 'wc_avatax_address' === $field['type'] ) {
				$this->save_address_field( $field );
				unset( $fields[ $key ] );
			}
		}

		WC_Admin_Settings::save_fields( $fields );
	}

	private function get_field( $fields, $id)
	{
		foreach ( $fields as $key => $field ) {
			if ( isset( $field['id'], $field['type'] ) && $id === $field['id'] ) {
				return $field;
			}
		}
	}

	/**
	 * Save the custom address field.
	 *
	 * @since 1.0.0
	 * @param array $field The field definition.
	 */
	private function save_address_field( $field ) {

		$address = isset( $_POST[ $field['id'] ] ) ? wp_unslash( $_POST[ $field['id'] ] ) : array();

		/**
		 * Filter the address values before the final save.
		 *
		 * @since 1.0.0
		 * @param array $address {
		 *     The address values.
		 *
		 * @type string @address_1 The street address.
		 * @type string @city      The city name.
		 * @type string @state     The state.
		 * @type string @country   The country code.
		 * @type string @postcode  The postal code.
		 * }
		 */
		$address = (array) apply_filters( 'wc_avatax_save_address_field', $address );

		$address = array_map( 'wc_clean', $address );

		if ( ! empty( $address ) ) {
			update_option( $field['id'], $address );
		}
	}

	/**
	 * Gets the product sync settings fields.
	 *
	 * @since 1.13.0
	 *
	 * @return array
	 */
	protected function get_product_sync_settings() : array {

		$settings = [];
		if (wc_avatax()->has_api_credentials_set() && wc_avatax()->check_api() && wc_avatax()->wc_avatax_utilities()->has_nexus_outside_USandCanada() ) {
			$settings = [
				[
					'type' => 'title',
					'name' => __( 'Transactions outside the US', 'woocommerce-avatax' ),
				],
				[
					'id'      => 'wc_avatax_enable_vat',
					'title'   => __( 'Collect Business VAT ID', 'woocommerce-avatax' ),
					'desc_tip'    => __( 'Turn this on to provide a place for customers to enter their Business VAT ID during checkout.', 'woocommerce-avatax' ),
					'type'    => 'checkbox',
					'class'	   => 'checkbox-toggle',
					'default' => 'yes',
				],
				[
					'id'       => 'wc_avatax_enable_cross_border_classification',
					'name'     => __( 'Calculate customs duties', 'woocommerce-avatax' ),
					'desc_tip'     => __( 'Enable to calculate customs duties for orders outside of the US. In AvaTax, line items must have a fully qualified tariff code to calculate customs duties for the destination country.', 'woocommerce-avatax' ),
					'default'  => 'no',
					'type'     => 'checkbox',
					'class'	   => 'checkbox-toggle',
				],
				[
					'id'       => 'wc_avatax_hs_api_username',
					'name'     => __( 'AvaTax Username', 'woocommerce-avatax' ),
					'type'     => 'text',
					'css'      => 'min-width:300px;',
					'desc' => __( 'The username used for your Avalara Login.', 'woocommerce-avatax' ),
				],
				[
					'id'       => 'wc_avatax_hs_api_password',
					'name'     => __( 'AvaTax password', 'woocommerce-avatax' ),
					'type'     => 'password',
					'css'      => 'min-width:300px;',
					'desc' => __( 'The password used for your Avalara Login.', 'woocommerce-avatax' ),
				],
				[
					'id'                => 'wc_avatax_api_product_countries_sync',
					'name'              => __( 'Countries for HS Code assignment', 'woocommerce-avatax' ),
					'type'              => 'multiselect',
					'class'             => 'wc-enhanced-select',
					'options'           => $this->get_normalized_supported_countries_options(),
					'custom_attributes' => [ 'multiple' => 'multiple' ],
					'desc'              => __( 'Select countries outside of the US where you would like to calculate the customs duties. Avalara automatically assigns HS Codes for the countries you select.', 'woocommerce-avatax' ),
				],
				[
					'id'       => 'wc_avatax_api_product_sync',
					'name'     => __( 'Sync products with Avalara', 'woocommerce-avatax' ),
					'type'     => 'wc_avatax_product_sync',
					'desc'     => $this->get_cross_border_product_sync_description(),
				],
				[ 'type' => 'sectionend' ],
			];
		}

		/**
		 * Filters the product sync settings fields.
		 *
		 * @since 1.13.0
		 *
		 * @param array $settings product sync fields
		 */
		return (array) apply_filters( 'woocommerce_get_settings_' . $this->id . '_product_sync', $settings );
	}

	/**
	 * Displays the API License field with landing message and connect, disconnect, test connection button.
	 *
	 * @since 2.7.0
	 *
	 * @return array
	 */
	public function display_api_license_key_field( $options ) {

		$id           = $options['id'];
		$label        = $options['title'];
		$connected    = 'connected' === get_transient( 'wc_avatax_connection_status', "not-connected" );
		$value		  = $options['value'];
		$company_name = get_transient("wc_avatax_company_name");
		$env 		  = get_option('wc_avatax_api_environment', 'production');
		$message = sprintf( __( 'If you are new to Avalara, %1$ssign up%2$s to generate your account information.', 'woocommerce-avatax' ),
			'<a href="https://admin-avatax.avalara.net" target="_blank">',
			'</a>'
		);
		$connected_message = __('<p>Your WooCommerce store is connected to <b>'.$company_name.'</b> in Avalara. <br /><br /> Go <a href="'. ($env == 'production' ? 'https://integrations.avalara.com/' : 'https://sandbox.integrations.avalara.com/') . '" target="_blank"> back to Avalara</a> to finish setting up the tax profile for <b>'.$company_name.'</b>.</p><button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button><span class="info-icon">i</span>', 'woocommerce-avatax');
		require_once( $this->get_plugin()->get_plugin_path() . '/src/admin/views/html-field-api-license-key.php' );
	}

	/**	
	 * Displays the API environment field.
	 *
	 * @since 2.7.0
	 */
	public function display_api_environment_field( $options ) {
		$value		  = get_option("wc_avatax_api_environment");
		require_once( $this->get_plugin()->get_plugin_path() . '/src/admin/views/html-field-api-environment.php' );
	}

	/**	
	 * Gets the Nexus enabled countries.
	 *
	 * @since 2.7.0
	 *
	 * @return array
	 */
	private function get_supported_countries() : array {
		$supported_countries =[];
		$supported_countries =  get_transient( 'wc_avatax_supported_countries_list');
		if(empty( $supported_countries ))
		{
			$cache_expiration = apply_filters( 'wc_avatax_connection_status_cache_expiration', HOUR_IN_SECONDS * 1 );
			$supported_countries = $this->get_plugin()->get_landed_cost_handler()->get_supported_countries();
			set_transient( 'wc_avatax_supported_countries_list', $supported_countries, $cache_expiration );
		}
		return $supported_countries;
	}



	/**	
	 * Fetches ECM enabled Countries from Nexus.
	 *
	 * @since 2.7.0
	 *
	 * @return array
	 */
	public function get_ecm_enabled_countries(): array {
		$supported_countries =  get_transient( 'wc_avatax_supported_countries_list');
		if($supported_countries == null)
		{
			$supported_countries = $this->get_supported_countries();
		}
		$ecm_enabled_countries = [
			(in_array('US', $supported_countries) ? 'US' : ''),
			(in_array('CA', $supported_countries)  ? 'CA' : '')
		];
		array_filter($ecm_enabled_countries);
		return $ecm_enabled_countries;
	}

	/**
	 * Gets a list of countries that’s ready to be used as a setting option.
	 *
	 * @since 1.13.0
	 *
	 * @return array
	 */
	private function get_normalized_supported_countries_options() : array {

		$this->cache_company_id();
		$supported_countries  = array_map( 'mb_strtoupper', $this->get_supported_countries() );

		$normalized_countries = ( new WC_Countries() )->get_countries();

		foreach ( $normalized_countries as $country_code => $country_name ) {
			if ( ! in_array( $country_code, $supported_countries, true ) ) {
				unset( $normalized_countries[ $country_code ] );
			}
		}

		return $normalized_countries;
	}

	/**
	 * Gets the Cross Border Product Sync field description.
	 *
	 * @since 1.13.0
	 *
	 * @return string
	 */
	private function get_cross_border_product_sync_description() : string {

		if ( $this->get_plugin()->get_landed_cost_sync_handler()->is_syncing_active() ) {
			return __( 'Sending products for classification! It may take some time for classification to complete. You can safely leave this page.', 'woocommerce-avatax' );
		}

		$sync_error_items = '';

		if ( ! $this->get_plugin()->has_hs_api_credentials_set() ) {
			$sync_error_items .= '<li>' . __( 'Please fill in your Avalara username and password', 'woocommerce-avatax' ) . '</li>';
		}

		if ( ! $this->get_plugin()->get_landed_cost_handler()->has_countries_for_product_sync() ) {
			$sync_error_items .= '<li>' . __( 'Please select at least one supported country.', 'woocommerce-avatax' ) . '</li>';
		}

		return ! empty( $sync_error_items ) ? sprintf(
			/* translators: Placeholders: %1$s - opening <strong> HTML tag, %2$s - closing </strong> HTML tag, %3$s - unordered HTML list of sync errors */
			__( '%1$sUnable to sync. Please correct the following errors:%2$s %3$s', 'woocommerce-avatax' ),
			'<strong>',
			'</strong>',
			'<ul class="wc-avatax-producst-sync-errors ul-disc" style="color: #A94545;">' . $sync_error_items . '<ul>'
		) : '';
	}

	/**
	 * Displays a custom product sync settings field.
	 *
	 * @internal
	 *
	 * @since 1.13.0
	 *
	 * @param array $options
	 */
	public function display_product_sync_field( $options ) {

		$description = WC_Admin_Settings::get_field_description( $options );

		$id           = $options['id'];
		$label        = $options['title'];
		$tooltip_html = $description['tooltip_html'];
		$description  = $description['description'];
		$disabled     = ! $this->is_cross_border_product_sync_connection_allowed();
		$syncing      = $this->get_plugin()->get_landed_cost_sync_handler()->is_syncing_active();

		require_once( $this->get_plugin()->get_plugin_path() . '/src/admin/views/product-sync.php' );
	}

	/**
	 * Handles the sync countries list update.
	 *
	 * This is a callback to be executed when the list of countries to sync is updated.
	 *
	 * @internal
	 *
	 * @since 1.13.0
	 *
	 * @param array $old_countries
	 * @param array $new_countries
	 */
	public function handle_countries_sync_update( $old_countries, $new_countries ) {

		$sync_handler = $this->get_plugin()->get_landed_cost_sync_handler();

		if ( $sync_handler->is_syncing_active() && ! empty( array_diff( $new_countries, $old_countries ) ) ) {
			$sync_handler->enqueue_full_sync();
		}
	}

	/**
	 * Determines whether the API environment is set to production.
	 *
	 * @since 1.13.0
	 * @deprecated 1.16.0
	 *
	 * @return bool
	 */
	public function is_api_environment_production() : bool {

		wc_deprecated_function( __METHOD__, '1.16.0' );

		return 'production' === get_option( 'wc_avatax_api_environment' );
	}

	/**
	 * Determines whether the store may try to connect to Cross-Border product sync.
	 *
	 * @since 1.13.0
	 *
	 * @return bool
	 */
	public function is_cross_border_product_sync_connection_allowed() : bool {

		$connection_allowed =
			wc_avatax()->has_api_credentials_set()
			&& wc_avatax()->has_hs_api_credentials_set()
			&& wc_avatax()->get_landed_cost_handler()->has_countries_for_product_sync();

		/**
		 * Filters whether the user is allowed to connect Cross Border product sync.
		 *
		 * @since 1.13.0
		 *
		 * @param bool $connection_allowed whether the user is allowed to connect Cross Border product sync
		 */
		return (bool) apply_filters( 'wc_avatax_cross_border_product_sync_connection_allowed', $connection_allowed );
	}

	/**
	 * Gets an instance of the plugin main class.
	 *
	 * @since 1.13.0
	 *
	 * @return WC_AvaTax
	 */
	protected function get_plugin() : WC_AvaTax {

		return wc_avatax();
	}

	/**
	 * Caches the company ID to prevent race conditions with other requests that depend on it.
	 *
	 * @since 1.13.0
	 */
	protected function cache_company_id() {

		wc_avatax()->get_company_id();
	}

	/**
	 * Clears the company ID cache when updating the company code.
	 *
	 * Also stops sync in progress if the company code changes when saving settings.
	 *
	 * @internal
	 *
	 * @since 1.17.0
	 *
	 * @param string|mixed $old_company_code
	 * @param string|mixed $new_company_code
	 * @return void
	 */
	public function prune_company_id_cache( $old_company_code, $new_company_code ) {

		if ( $old_company_code === $new_company_code ) {
			return;
		}
		wc_avatax()->clear_company_id_cache();
		wc_avatax()->get_landed_cost_sync_handler()->stop_syncing();
		add_action( 'woocommerce_settings_saved', [ $this, 'save_company_default_fields' ] );
	}

	/**
	 * Save default fields. Enables all visible settings when connecting to AvaTAx.
	 *
	 * @internal
	 *
	 * @since 2.7.0
	 * 
	 */
	public function save_company_default_fields()
	{
		wc_avatax()->wc_avatax_utilities()->save_company_default_fields();
	}
	/**
	 * Clears the account number cache when updating the account number.
	 *
	 * Also stops sync in progress if the account number changes when saving settings.
	 *
	 * @internal
	 *
	 * @since 2.3.0
	 *
	 * @param string|mixed $old_account_number
	 * @param string|mixed $new_account_number
	 * @return void
	 */
	public function prune_account_number_cache( $old_account_number, $new_account_number ) {

	if ($old_account_number=== $new_account_number) {
			return;
		}
		wc_avatax()->get_landed_cost_sync_handler()->stop_syncing();
		wc_avatax()->clear_account_number_cache();
	}

	

	/**
	 * Saves origin Address Field
	 *
	 * @internal
	 *
	 * @since 2.7.0
	 *
	 * @return void
	 */
	public function save_origin_address_field() {
		if(wc_avatax()->has_api_credentials_set() && wc_avatax()->check_api())
		{
			$response = wc_avatax()->get_api()->get_company_location();
			if ($response != null)
			{
				$applicable_address = array(
					'address_1' =>  $response->line1,
					'country'  => $response->country ,
					'state'    => $response->region ,
					'postcode' =>  $response->postalCode ,
					'city'     =>  $response->city
				);
				update_option('wc_avatax_origin_address', $applicable_address);
			}
			else {
				if(WC()->customer != null) {
					$applicable_address = array(
						'address_1' => WC()->customer->get_shipping_address(),
						'country'  => WC()->customer->get_shipping_country(),
						'state'    => WC()->customer->get_shipping_state(),
						'postcode' => WC()->customer->get_shipping_postcode(),
						'city'     => WC()->customer->get_shipping_city()
					);
					update_option('wc_avatax_origin_address', $applicable_address);
				}
			}
		}
	}
}
