<?php
/**
 * WooCommerce AvaTax
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@skyverge.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce AvaTax to newer
 * versions in the future. If you wish to customize WooCommerce AvaTax for your
 * needs please refer to http://docs.woocommerce.com/document/woocommerce-avatax/
 *
 * @author    SkyVerge
 * @copyright Copyright (c) 2016-2022, SkyVerge, Inc.
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

defined( 'ABSPATH' ) or exit;

use SkyVerge\WooCommerce\AvaTax\Api\WC_AvaTax_HS_API;
use SkyVerge\WooCommerce\AvaTax\Landed_Cost_Sync_Handler;
use SkyVerge\WooCommerce\PluginFramework\v5_10_14 as Framework;

/**
 * WooCommerce AvaTax main plugin class.
 *
 * @since 2.7.0
 */
class WC_AvaTax_Utilities {

	/* ECM Subscription names */
	const TYPE_AVATAX_ECMESSENTIALS = 'ECMEssentials';
	const TYPE_AVATAX_ECMPRO = 'ECMPro';
	const TYPE_AVATAX_ECMPREMIUM = 'ECMPremium';
    
    /**
     * Disconnects the connection to AvaTax.
     * 
     * @since 2.7.0
     *
     */
	public static function disconnect_avatax($is_update = false) {
        
		if(!$is_update) {
			update_option("wc_avatax_api_environment", '' );
			update_option("wc_avatax_api_account_number", '');
			update_option("wc_avatax_api_license_key", '');
		}
		
		global $wpdb;
		$wpdb->query( "DELETE FROM $wpdb->options WHERE option_name in ('wc_avatax_enable_tax_calculation','wc_avatax_record_calculations', 'wc_avatax_company_code', 'wc_avatax_enable_co_rdf', 'wc_avatax_enable_ecm', 'wc_avatax_enable_vat', 'wc_avatax_enable_cross_border_classification', 'wc_avatax_origin_address', 'wc_avatax_hs_api_username', 'wc_avatax_hs_api_password', 'wc_avatax_api_product_countries_sync', 'wc_avatax_debug', 'wc_avatax_enable_address_validation', 'wc_avatax_supported_countries_list' )" );
		$wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE '_transient_wc_avatax_%'" );
		$wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE '_transient_timeout_wc_avatax_%'" );

		
	}

	/**
     * Checks whether the Colorado nexus is present in AvaTax.
     * 
     * @since 2.7.0
     *
     */
	public function is_colorado_nexus_present(){
		$supported_states = wc_avatax()->get_landed_cost_handler()->get_supported_states('US');
			return in_array('CO', $supported_states);
	}

	/**
     * Get the subscriptions for current account from AvaTax.
     * 
     * @since 2.7.0
     *
     */
	protected function get_subscriptions() {
		$subscriptions_list = [];
		$subscriptions_list = get_transient( 'wc_avatax_subscriptions_list' );
		if($subscriptions_list == null)
		{
			$cache_expiration = apply_filters( 'wc_avatax_connection_status_cache_expiration', HOUR_IN_SECONDS * 1 );
			$subscriptions_list = wc_avatax()->get_api()->get_subscriptions()->get_subscriptions();
			set_transient( 'wc_avatax_subscriptions_list', $subscriptions_list, $cache_expiration );
		}
		return $subscriptions_list;
	}

	/**
     * Checks whether the subscription is present or not.
     * 
     * @since 2.7.0
	 * @param string[] array of subscription names
	 * @return bool
     */
	public function has_subscription(array $type ) : bool {
		$subscriptions =  wp_list_pluck( get_transient( 'wc_avatax_subscriptions_list'), 'subscriptionDescription' );
		if($subscriptions == null && wc_avatax()->has_api_credentials_set() && wc_avatax()->check_api())
		{
			$subscriptions = $this->get_subscriptions();
			$subscriptions = wp_list_pluck( $subscriptions , 'subscriptionDescription' );
		}
		return count(array_intersect($subscriptions, $type)) ? true : false;
	}
    
	/**
	 * Checks wheter account has ECM subscription or not.
	 *
	 * @since 2.7.0
	 *
	 * @return bool
	 */
	public function has_ecm_subscription() {
		return $this->has_subscription( array(self::TYPE_AVATAX_ECMPRO, self::TYPE_AVATAX_ECMESSENTIALS, self::TYPE_AVATAX_ECMPREMIUM));
	}

	/**	
	 * Gets the Nexus enabled countries.
	 *
	 * @since 2.7.0
	 *
	 * @return array
	 */
	private function get_supported_countries() : array {
		$supported_countries =[];
		$supported_countries =  get_transient( 'wc_avatax_supported_countries_list');
		if(empty( $supported_countries ))
		{
			$cache_expiration = apply_filters( 'wc_avatax_connection_status_cache_expiration', HOUR_IN_SECONDS * 1 );
			$supported_countries = wc_avatax()->get_landed_cost_handler()->get_supported_countries();
			set_transient( 'wc_avatax_supported_countries_list', $supported_countries, $cache_expiration );
		}
		return $supported_countries;
	}

	/**
	 * Save default fields. Enables all visible settings when connecting to AvaTAx.
	 *
	 * @internal
	 *
	 * @since 2.7.0
	 * 
	 */
	public function save_default_fields() {
		if(wc_avatax()->has_api_credentials_set() && wc_avatax()->check_api())
		{
			update_option('wc_avatax_default_product_code', 'P0000000');
			update_option('wc_avatax_shipping_code', 'FR');
			update_option('wc_avatax_enable_tax_calculation', 'yes');
			update_option('wc_avatax_record_calculations', 'yes');
			update_option('wc_avatax_debug', 'yes');
			$this->save_company_default_fields();
			if(wc_avatax()->wc_avatax_utilities()->has_ecm_subscription()) {
				update_option('wc_avatax_enable_ecm', 'yes');
			}

			if($this->has_nexus_outside_USandCanada()){
				update_option('wc_avatax_api_product_countries_sync',  $this->get_supported_countries());
			}
		}
	}
		/**	
	 * Checks whether the nexus is outside US/Canada.
	 *
	 * @since 2.7.0
	 *
	 * @return bool
	 */
	public function has_nexus_outside_USandCanada() : bool {
		$supported_countries =  get_transient( 'wc_avatax_supported_countries_list');
		if($supported_countries == null)
		{
			$supported_countries = $this->get_supported_countries();
		}
		return count($supported_countries) > count(array_intersect($supported_countries, ["US","CA"])) ? true : false;
	}
	/**
	 * Save default company fields. Enables all visible settings when connecting to AvaTAx.
	 *
	 * @internal
	 *
	 * @since 2.7.0
	 * 
	 */
	public function save_company_default_fields()
	{
		wc_avatax()->log("Inside save company default fields : ");
		if(wc_avatax()->wc_avatax_utilities()->is_colorado_nexus_present()){
			update_option('wc_avatax_enable_co_rdf', 'yes');
		}

		if(!$this->has_nexus_outside_USandCanada()){
			wc_avatax()->log("Inside save company default fields 1 : ");
			update_option('wc_avatax_enable_address_validation', 'yes');

		}
		else {
				update_option('wc_avatax_api_product_countries_sync',  $this->get_supported_countries());
				update_option('wc_avatax_enable_vat', 'yes');
		}
	}

	/**
	 * Checks if HPOS feature is enabled or not.
	 *
	 * @internal
	 *
	 * @since 2.7.1
	 * 
	 */
	public function is_hpos_enabled(){
		return (class_exists( Automattic\WooCommerce\Utilities\OrderUtil::class ) && Automattic\WooCommerce\Utilities\OrderUtil::custom_orders_table_usage_is_enabled());
	}

	/**
	 * Gets the order meta.
	 *
	 * @internal
	 *
	 * @param string $order_id the order id
	 * @param string $meta_key a metadata key
	 * @return string the found value for provided key
	 * 
	 * @since 2.7.1
	 * 
	 */
	public function get_order_meta($order_id, $meta_key, $single = true){
		if($this->is_hpos_enabled()) {
			$order = wc_get_order( $order_id );
			return $order->get_meta($meta_key, $single );
		}
		else{
			return get_post_meta( $order_id, $meta_key, $single );
		}
	}

	/**
	 * Adds the order meta data.
	 *
	 * @internal
	 *
	 * @param string $order_id the order id
	 * @param string $meta_key a metadata key
	 * @param string $meta_value a metadata value
	 * @return bool 
	 * 
	 * @since 2.7.1
	 * 
	 */
	public function add_order_meta($order_id, $meta_key, $meta_value){

		if($this->is_hpos_enabled()) {
			$order = wc_get_order( $order_id );
			$order->add_meta_data($meta_key, $meta_value);
			$order->save();
			return true;
		} else {
			// Traditional CPT-based orders are in use.
			return add_post_meta( $order_id, $meta_key, $meta_value );
		}
	}

	/**
	 * Updates the order meta data.
	 *
	 * @internal
	 *
	 * @param string $order_id the order id
	 * @param string $meta_key a metadata key
	 * @param string $meta_value a metadata value
	 * 
	 * @since 2.7.1
	 * 
	 */
	public function update_order_meta($order_id, $meta_key, $meta_value){

		if($this->is_hpos_enabled()) {
			$order = wc_get_order( $order_id );
			$order->update_meta_data($meta_key, $meta_value );
			$order->save();
		}
		else{
			update_post_meta( $order_id, $meta_key, $meta_value );
		}
	}

	/**
	 * Deletes the order meta data.
	 *
	 * @internal
	 *
	 * @param string $order_id the order id
	 * @param string $meta_key a metadata key
	 * @param string $meta_value a metadata value
	 * @return bool 
	 * 
	 * @since 2.7.1
	 * 
	 */
	public function delete_order_meta($order_id, $meta_key, $meta_value){
		if ( $this->is_hpos_enabled() ) {
			// HPOS usage is enabled.
			$order = wc_get_order( $order_id );
			$order->delete_meta_data($meta_key, $meta_value );
			$order->save();
			return true;
		} else {
			// Traditional CPT-based orders are in use.
			return delete_post_meta( $order_id, $meta_key, $meta_value );
		}
	}

	/**
	 * Gets the order type.
	 *
	 * @internal
	 *
	 * @param string $order_id the order id
	 * @return string 
	 * 
	 * @since 2.7.1
	 * 
	 */
	public function get_post_type($order_id){
		if($this->is_hpos_enabled()) {
			return Automattic\WooCommerce\Utilities\OrderUtil::get_order_type( $order_id );
		}
		else{
			return get_post_type( $order_id );
		}
	}

	/**
	 * Hide our custom line item meta from the order admin.
	 *
	 * @internal
	 *
	 * @since 2.7.1
	 *
	 * @param array $hidden_meta The hidden line item keys.
	 * @return array $hidden_meta
	 */
	public function hide_order_item_meta( $hidden_meta ) {

		$hidden_meta[] = '_wc_avatax_code';
		$hidden_meta[] = '_wc_avatax_rate';
		$hidden_meta[] = '_wc_avatax_hs_code';

		return $hidden_meta;
	}
}
